@@ -541,7 +541,7 @@ def split_ref_from_uri(uri):
     parsed = urllib_parse.urlparse(uri)
     path = parsed.path
     ref = None
-    if "@" in path:
+    if parsed.scheme != "file" and "@" in path:
         path, _, ref = path.rpartition("@")
     parsed = parsed._replace(path=path)
     return (urllib_parse.urlunparse(parsed), ref)
@@ -829,7 +829,9 @@ def name_from_req(req):
         return req.name
 
 
-def make_install_requirement(name, version, extras, markers, constraint=False):
+def make_install_requirement(
+    name, version=None, extras=None, markers=None, constraint=False
+):
     """
     Generates an :class:`~pip._internal.req.req_install.InstallRequirement`.
 
@@ -853,19 +855,16 @@ def make_install_requirement(name, version, extras, markers, constraint=False):
     from pip_shims.shims import install_req_from_line
 
     extras_string = ""
+    requirement_string = "{0}".format(name)
     if extras:
         # Sort extras for stability
         extras_string = "[{}]".format(",".join(sorted(extras)))
-
-    if not markers:
-        return install_req_from_line(
-            str("{}{}=={}".format(name, extras_string, version)), constraint=constraint
-        )
-    else:
-        return install_req_from_line(
-            str("{}{}=={}; {}".format(name, extras_string, version, str(markers))),
-            constraint=constraint,
-        )
+        requirement_string = "{0}{1}".format(requirement_string, extras_string)
+    if version:
+        requirement_string = "{0}=={1}".format(requirement_string, str(version))
+    if markers:
+        requirement_string = "{0}; {1}".format(requirement_string, str(markers))
+    return install_req_from_line(requirement_string, constraint=constraint)
 
 
 def version_from_ireq(ireq):
@@ -968,6 +967,24 @@ def get_name_variants(pkg):
     return names
 
 
+def read_source(path, encoding="utf-8"):
+    # type: (S, S) -> S
+    """
+    Read a source file and get the contents with proper encoding for Python 2/3.
+
+    :param AnyStr path: the file path
+    :param AnyStr encoding: the encoding that defaults to UTF-8
+    :returns: The contents of the source file
+    :rtype: AnyStr
+    """
+    if six.PY3:
+        with open(path, "r", encoding=encoding) as fp:
+            return fp.read()
+    else:
+        with open(path, "r") as fp:
+            return fp.read()
+
+
 SETUPTOOLS_SHIM = (
     "import setuptools, tokenize;__file__=%r;"
     "f=getattr(tokenize, 'open', open)(__file__);"
